@@ -1,7 +1,8 @@
+# frozen_string_literal: false
 #
 # extconf.rb
 #
-# $Id: extconf.rb 36939 2012-09-09 14:01:53Z naruse $
+# $Id: extconf.rb 53143 2015-12-16 05:31:54Z naruse $
 #
 
 require 'mkmf'
@@ -10,18 +11,16 @@ require 'rbconfig'
 dir_config 'zlib'
 
 
-if %w'z libz zlib1 zlib zdll'.find {|z| have_library(z, 'deflateReset')} and
+if %w'z libz zlib1 zlib zdll zlibwapi'.find {|z| have_library(z, 'deflateReset')} and
     have_header('zlib.h') then
 
   defines = []
 
-  message 'checking for kind of operating system... '
+  Logging::message 'checking for kind of operating system... '
   os_code = with_config('os-code') ||
     case RUBY_PLATFORM.split('-',2)[1]
     when 'amigaos' then
       os_code = 'AMIGA'
-    when /\Aos2[\-_]emx\z/ then
-      os_code = 'OS2'
     when /mswin|mingw|bccwin/ then
       # NOTE: cygwin should be regarded as Unix.
       os_code = 'WIN32'
@@ -36,7 +35,6 @@ if %w'z libz zlib1 zlib zdll'.find {|z| have_library(z, 'deflateReset')} and
     'OS_VMS'     => 'VMS',
     'OS_UNIX'    => 'Unix',
     'OS_ATARI'   => 'Atari',
-    'OS_OS2'     => 'OS/2',
     'OS_MACOS'   => 'MacOS',
     'OS_TOPS20'  => 'TOPS20',
     'OS_WIN32'   => 'Win32',
@@ -50,7 +48,7 @@ if %w'z libz zlib1 zlib zdll'.find {|z| have_library(z, 'deflateReset')} and
   unless OS_NAMES.key? os_code then
     raise "invalid OS_CODE `#{os_code}'"
   end
-  message "#{OS_NAMES[os_code]}\n"
+  Logging::message "#{OS_NAMES[os_code]}\n"
   defines << "OS_CODE=#{os_code}"
 
   $defs.concat(defines.collect{|d|' -D'+d})
