@@ -2,7 +2,7 @@
 
   vm.c -
 
-  $Author: kosaki $
+  $Author: nagachika $
 
   Copyright (C) 2004-2007 Koichi Sasada
 
@@ -376,7 +376,7 @@ vm_set_main_stack(rb_thread_t *th, const rb_iseq_t *iseq)
     vm_set_eval_stack(th, iseq, 0, &env->block);
 
     /* save binding */
-    if (bind && iseq->body->local_size > 0) {
+    if (iseq->body->local_size > 0) {
 	bind->env = vm_make_env_object(th, th->cfp);
     }
 }
@@ -468,20 +468,25 @@ rb_frame_pop(void)
 void
 ruby_vm_at_exit(void (*func)(rb_vm_t *))
 {
-    rb_ary_push((VALUE)&GET_VM()->at_exit, (VALUE)func);
+    rb_vm_t *vm = GET_VM();
+    rb_at_exit_list *nl = ALLOC(rb_at_exit_list);
+    nl->func = func;
+    nl->next = vm->at_exit;
+    vm->at_exit = nl;
 }
 
 static void
 ruby_vm_run_at_exit_hooks(rb_vm_t *vm)
 {
-    VALUE hook = (VALUE)&vm->at_exit;
+    rb_at_exit_list *l = vm->at_exit;
 
-    while (RARRAY_LEN(hook) > 0) {
-	typedef void rb_vm_at_exit_func(rb_vm_t*);
-	rb_vm_at_exit_func *func = (rb_vm_at_exit_func*)rb_ary_pop(hook);
+    while (l) {
+	rb_at_exit_list* t = l->next;
+	rb_vm_at_exit_func *func = l->func;
+	free(l);
+	l = t;
 	(*func)(vm);
     }
-    rb_ary_free(hook);
 }
 
 /* Env */
@@ -917,15 +922,12 @@ invoke_block(rb_thread_t *th, const rb_iseq_t *iseq, VALUE self, const rb_block_
 }
 
 static VALUE
-invoke_bmethod(rb_thread_t *th, const rb_iseq_t *iseq, VALUE self, const rb_block_t *block, int type, int opt_pc)
+invoke_bmethod(rb_thread_t *th, const rb_iseq_t *iseq, VALUE self, const rb_block_t *block, const rb_callable_method_entry_t *me, int type, int opt_pc)
 {
     /* bmethod */
     int arg_size = iseq->body->param.size;
-    const rb_callable_method_entry_t *me = th->passed_bmethod_me;
     VALUE ret;
 
-    th->passed_bmethod_me = NULL;
-
     vm_push_frame(th, iseq, type | VM_FRAME_FLAG_FINISH | VM_FRAME_FLAG_BMETHOD, self,
 		  VM_ENVVAL_PREV_EP_PTR(block->ep),
 		  (VALUE)me, /* cref or method (TODO: can we ignore cref?) */
@@ -954,6 +956,9 @@ invoke_block_from_c_0(rb_thread_t *th, const rb_block_t *block,
 	int i, opt_pc;
 	int type = block_proc_is_lambda(block->proc) ? VM_FRAME_MAGIC_LAMBDA : VM_FRAME_MAGIC_BLOCK;
 	VALUE *sp = th->cfp->sp;
+	const rb_callable_method_entry_t *me = th->passed_bmethod_me;
+
+	th->passed_bmethod_me = NULL;
 
 	for (i=0; i<argc; i++) {
 	    sp[i] = argv[i];
@@ -962,11 +967,11 @@ invoke_block_from_c_0(rb_thread_t *th, const rb_block_t *block,
 	opt_pc = vm_yield_setup_args(th, iseq, argc, sp, blockptr,
 				     (type == VM_FRAME_MAGIC_LAMBDA ? (splattable ? arg_setup_lambda : arg_setup_method) : arg_setup_block));
 
-	if (th->passed_bmethod_me == NULL) {
+	if (me == NULL) {
 	    return invoke_block(th, iseq, self, block, cref, type, opt_pc);
 	}
 	else {
-	    return invoke_bmethod(th, iseq, self, block, type, opt_pc);
+	    return invoke_bmethod(th, iseq, self, block, me, type, opt_pc);
 	}
 
     }
@@ -2170,8 +2175,6 @@ vm_init2(rb_vm_t *vm)
     MEMZERO(vm, rb_vm_t, 1);
     rb_vm_living_threads_init(vm);
     vm->src_encoding_index = -1;
-    vm->at_exit.basic.flags = (T_ARRAY | RARRAY_EMBED_FLAG) & ~RARRAY_EMBED_LEN_MASK; /* len set 0 */
-    rb_obj_hide((VALUE)&vm->at_exit);
 
     vm_default_params_setup(vm);
 }
